#!/usr/bin/env perl
#
#  identify2.pl
#  2004-04-20 stvo 
#  $Id: identify2.pl,v 1.9 2017/05/15 12:33:50 testx Exp testx $
#
use warnings;
use strict;
use Digest::MD5;
use Data::Dumper;

my $BASEPATH = "/usr/local/pd-admin2";
die "cannot automatically update a system w/o VERSION file.\n"
   . "Please contact providerdienste.de.\n"
   unless (-f $BASEPATH . '/UPDATE.INF/VERSION');

$| = 1;

#
#  print level
#
my $PRLEV = 1;

my $files2 = { };
sub readin2 ($$)
{
   my $has_data2 = 0;
   my @fil2;

   my $fil = shift;
   my $fil2 = shift;

   if (-f $fil2) {
      print "$fil2 found, using fast file check\n";
      open (FI2, '<', $fil2) or die "open $fil2: $!";
      @fil2 = <FI2>;
      chomp @fil2;
      close (FI2);
#      print "fil2: ", scalar @fil2, "\n";
   }
   else {
      print "Using old (slow) checksum based file check.\n";
   }

   die "need at least $fil\n" unless -f $fil;

   open (FI, '<', $fil) or die "open $fil: $!";
   my ($size, $ts) = (undef, undef);
   my $indx = -1;
   while (my $l = <FI>) {
      ++$indx;
      next if ($l =~ /^\s*#/);
      chomp $l;
      if (not $l =~ /^ *#/) {
         my ($ind, $digest, $obj) = split / /, $l, 3
            or die "ERROR: $l\n";
         if (@fil2) {
            ($size, $ts) = split /,/, $fil2[$indx], 2
               or die "ERROR: @fil2\n";
         }
#         print "<$ind> <$digest> <$obj> <$size> <$ts>\n";
         die "obj not defined, $_" if not defined $obj;
         die "digest not defined, $_" if not defined $digest;
         die "ind not defined, $_" if not defined $ind;
         die "ind not 2 figures, $ind" if not (length ($ind) == 2);
         die "digest not 22 figures, $ind" if not (length ($digest) == 22);
#         die "size not defined" unless defined $size;
#         die "timestamp not defined" unless defined $ts;

         if ($l =~ /^OD/) {
            die "should not land here: ", __FILE__, ":", __LINE__;
         }
         else {
            $files2->{$obj} = [$size, $ts, $digest];
         }
      }
      else {
         print $l, "\n";
         die "should not land here: ", __FILE__, ":", __LINE__;
      }
   }
   print "\n";
}

my $md5 = Digest::MD5->new;
sub get_digest ($)
{
   my $obj = shift;
   $md5->reset;
   my $digest;
   undef $digest;

#   print "OBJ = $obj\n";

   if (-l $obj) {
      my $v = readlink $obj or die "cannot read link $obj: $!";
      $md5->add ($v);
      $digest = $md5->b64digest ();
#      warn "$obj: <$digest>\n";
   }
   elsif (-f $obj) {
      open (FI, '<', $obj) or die "cannot open file $obj: $!";
      $md5->addfile (*FI);
      $digest = $md5->b64digest ();
      close FI;
   }
   elsif (-S $obj or -p $obj or -b $obj or -c $obj) {
      $digest = '0' x 22;
   }
   else {
      warn "unknown object type $obj";
   }
#   print "DIG = $digest\n";
   return $digest;
}

my @unchanged_files;
my @changed_files;
my @new_files;

sub file_identification ($)
{
   my $ndir = shift;

   unless (exists $files2->{$ndir}) {
#      warn "debug trace: $ndir\n";
      push @new_files, $ndir;
      return;
   }
   my $fullpath = $BASEPATH . $ndir;
   my ($size, $ts, $hash) = @{$files2->{$ndir}};
   if (defined $size and defined $ts) {
      my ($fssize, $fsts) = (lstat $fullpath) [7,9];
      die "fssize undefined" unless defined $fssize;
      die "fsts undefined" unless defined $fsts;
      if ($size != $fssize) {
         push @changed_files, $ndir;
         return;
      }
      if ($ts == $fsts) {
         push @unchanged_files, $ndir;
         return;
      }
   }
   if (defined $hash) {
#      print "hashing $fullpath\n";
      my $fshash = get_digest $fullpath;
      if ($hash eq $fshash) {
         push @unchanged_files, $ndir;
         return;
      }
      push @changed_files, $ndir;
      return;
   }
   die "hash not defined\n";
}

my $prefix = '';
my $indent = 0;
sub traversedir ($);
sub traversedir ($)
{
   my $dir = shift;
   ++$indent;

#   print "DIR=", $prefix . $dir, "\n";
   opendir (DIR, $prefix . $dir) or do {
      warn "cannot open ", $prefix . $dir, "\n";
      --$indent;
      return;
   };
   my @dirents = readdir DIR;
   closedir DIR;
#
#  process fs object in their natural order
#
   for my $d (sort @dirents) {
      next if ($d eq '.' or $d eq '..');
      my $ndir = $dir . '/' . $d;
      if (not -l $prefix . $ndir and -d $prefix . $ndir) {
         traversedir ($ndir);
      }
      else {
         file_identification ($ndir);
      }
   }
   --$indent;
}

sub main ()
{
   readin2
      $BASEPATH . '/UPDATE.INF/data.inf',
      $BASEPATH . '/UPDATE.INF/data2.inf';
#
#  2004-12-28 stvo
#  if hotfixes added some inf-files read them
#
   my @hff = glob ${BASEPATH} . '/UPDATE.INF/hotfix-*.inf';
   for my $hf (@hff) {
      print $hf, "\n";
      readin2 $hf, undef;
   }

   print "Bitte warten, Versionsidentifizierung luft ...\n";
   $prefix = $BASEPATH;
   traversedir ('');
   print "\nVersionsmuster\n";
   my $version = `cat $BASEPATH/UPDATE.INF/VERSION`;
   chomp $version;
   print "      SE $version . . . .: ", scalar @unchanged_files, "\n";

   if (-f 'VERSION') {
      unlink 'VERSION' or die "cannot unlink VERSION: $!";
   }
   open FO, ">VERSION" or die "cannot open VERSION: $!";
   my $dotlessversion = $version;
   $dotlessversion =~ s/\.//g;
   print FO $dotlessversion, "\n";
   close FO;

   print "Neue Dateien . . . . .: ", scalar @new_files, "\n";
   print "Genderte Dateien. . .: ", scalar @changed_files, "\n";
   print "\nGefunden . . . . . . .: SE $version\n";

   $PRLEV > 3 && do {
      print"\n=== UNCHANGED FILES ===\n";
         print scalar @unchanged_files, "\n";
      print"\n=== POSSIBLY CHANGED FILES ===\n\n";
      for my $i (@changed_files) {
         print $i, "\n";
      }
      print"\n=== NEW FILES ===\n\n";
      for my $i (@new_files) {
         print $i, "\n";
      }
   };
#
#     generate delete information 
#
   open FOD, ">OLDFILES" or die "cannot open OLDFILES: $!";
   $, = "\n";
   print FOD @unchanged_files, '';
   $, = "";
   close FOD;

   open FOD, ">CHANGEDFILES" or die "cannot open CHANGEDFILES: $!";
   $, = "\n";
   print FOD @changed_files, '';
   $, = "";
   close FOD;

   open FOD, ">NEWFILES" or die "cannot open NEWFILES: $!";
   $, = "\n";
   print FOD @new_files, '';
   $, = "";
   close FOD;
}

main ();
