#!/usr/bin/env perl
#
#  2004-04-22 stvo
#  unlink a list of files
#  remove empty diretories
#
#  $Id: massunlink.pl,v 1.8 2016/05/12 08:22:04 testx Exp testx $
#

use warnings;
use strict;
#use Data::Dumper;

my $remove_count = 0;
my $nremove_count = 0;
my %allowed_dirs;
my $no_delete_group = { };
sub remove_all_dirs ($);
sub remove_all_dirs ($)
{
   my $dir = shift;
   $dir = $dir . "/";
   if (opendir DIR, "$dir/") {
      my @nodes = readdir (DIR);
      closedir DIR;
      for my $i (@nodes) {
         if ($i ne '.' and $i ne '..') {
            lstat "$dir$i";
            if (! -l $dir . $i and -d "$dir$i") {
               remove_all_dirs ($dir . $i);
               if (defined $allowed_dirs{$dir . $i}) {
                  if (rmdir $dir . $i) {
                     ++$remove_count;
                  }
                  else {
                     ++$nremove_count;
                  }
               }
            }
         }
      }
   }
   else {
      warn "opendir: $!, directory is $dir\n";
   }
}

if (-l 'pd-admin2') {
   my $n = unlink 'pd-admin2';
   die "unlink 'pd-admin2' did not return 1: $!" unless $n == 1;
}

symlink '/usr/local/pd-admin2.work','pd-admin2'
   or die "cannot create symlink to working directory: $!";

#
#  fetch all directory names
#
if (-f "/usr/local/pd-admin2/UPDATE.INF/data.inf") {
   open (FI, "</usr/local/pd-admin2/UPDATE.INF/data.inf")
      or die "cannot read UPDATE.INF/data.inf: $!";
   while (<FI>) {
      chomp;
      if (/^DA 0000000000000000000000 (.*)$/) {
         $allowed_dirs{'./pd-admin2/' . $1} = 1;
      }
   }
   close (FI);
}
else {
   print "Will not delete any empty directory.\n"
}

#
#  fetch no delete groups (NDGs)
#
if (-f "ndgs.inf") {
   open (FI, "<ndgs.inf")
      or die "cannot read ./ndgs.inf: $!";
   my $ht = {};
   while (<FI>) {
      chomp;
      my ($num, $tex) = split / /;
#      print $num, ":", $tex, "\n";
      if (not exists $ht->{$num}) {
         $ht->{$num} = $tex;
      }
      else {
         $no_delete_group->{$tex} = $ht->{$num};
      }
   }
   close (FI);
#   print Dumper ($no_delete_group), "\n";
}
else {
   print "Will not delete any empty directory.\n"
}

open FI, "<$ARGV[0]" or die "cannot read $ARGV[0]\n";

while (<FI>) {
   chomp;
   if (not exists $no_delete_group->{$_}) {
      unlink "./pd-admin2/$_" or die "could not unlink ./pd-admin2/$_";
   }
   else {
      print "Not deleting ", $_, " (NDG ", $no_delete_group->{$_}, ")\n";
   }
}

close (FI);


#
#  remove all empty dirs
#
remove_all_dirs './pd-admin2/';
print "$remove_count directories removed\n";
print "$nremove_count directories not removed (not empty)\n";
