#!/bin/bash
#
#  2015-07-21 stvo
#  remove old .pdu-???? files, keep those beloging to current SE
#  version
#
#  $Id: pduclean.sh,v 1.14 2019/04/10 10:07:23 testx Exp $
#
BASE=/usr/local/pd-admin2
SERIES=$(cat ${BASE}/UPDATE.INF/SERIES)
VERSION=$(cat ${BASE}/UPDATE.INF/VERSION)
if ! echo $VERSION | egrep -q '^[[:digit:]]\.[[:digit:]]{3,3}$'; then
   echo "VERSION = <$VERSION> but must be in format a.bcd with a,b,c,d digits"
   exit 1
fi

#echo "VERSION = <$VERSION>"
#
#  remove dot
#
DOTLESSVERSION=${VERSION:0:1}${VERSION:2:3}
if [ ${#DOTLESSVERSION} -ne 4 ]; then
   echo "DOTLESSVERSION = <$DOTLESSVERSION> form error. Expected: abcd"
   exit 2
fi
#echo "DOTLESSVERSION = <$DOTLESSVERSION>"

type readlink 1> /dev/null || exit 8
READLINK_E="readlink -e"
$READLINK_E / &> /dev/null || READLINK_E="readlink -f"
$READLINK_E / > /dev/null || exit 9

cat <<_
Server Environment/Serverumgebung

   Series/Reihe  Release/Version  dotlessversion  READLINK_E
   ------------  ---------------  --------------  ----------------
   $SERIES             $VERSION            $DOTLESSVERSION            $READLINK_E

_


yn=

div()
{
   cat <<_
------------------------------------------------------------------------
_
}

multi_suffix ()
{
   test -z $yn && echo "dry run"
   deleted=0
   renamed=0
   while IFS= read -r -d $'\0' f; do
      echo 
      echo "f: $f"
      fbase=${f//.pdu-????}
      fm0=$f
      fm2=""
      while [ "$fm2" != "$fbase" ]; do
         fm1=${fm0/.pdu-????}
         fm2=${fm1/.pdu-????}
         if [ "$fm1" = "$fm" ]; then
            echo "$fm1, $fm2"
            echo "unexpected fail: need at least two pdu suffixes"
            exit 7
         fi
         echo "deleting $fm1"
         test -z $yn || unlink "$fm1"
         ((++deleted))
         fm0=$fm1
      done
      echo "renaming $f -> $fm1"
      test -z $yn || mv "$f" "$fm1"
      ((++renamed))
   done < <(find ${BASE} -xdev -name \*.pdu-????.pdu-$DOTLESSVERSION -print0)
   echo "deleted: $deleted"
   echo "renamed: $renamed"
}


activate ()
{
   div
   yn="" multi_suffix > /dev/null
   if test $renamed -gt 0; then
      echo "found $renamed file(s)"
      echo "please remove files with multiple suffixes first: "
      echo ""
      echo "     $0 -ym"
      echo ""
      exit 8
   fi
   tc=$OPTARG
   if test -L "$tc" -o -e "$tc"; then
#      echo "tc = <$tc> exists"
      :
   else
#      echo "2: tc = <$tc>"
      tc="$BASE/$tc"
#      echo "2: tc = <$tc>"
      if ! test -L "$tc" -o -e "$tc"; then
         echo "tc = <$tc> does not exist"
         exit 7
      fi
   fi

#
#  if the argument is a symbolic link protect the last path
#  component from being resolved (read) by readlink.
#
   if test -L "$tc"; then
      pathname=${tc%/*}
      linkname=${tc##*/}
      if [ "$pathname" = "$linkname" ]; then
         pathname="."
      fi
      linkname="/$linkname"
   else
      pathname=$tc
   fi
#   echo "pathname = <$pathname>, linkname = <$linkname>"

   if ! p=$($READLINK_E "$pathname"); then
      echo "1: p = <$p>, $pathname not found"
      exit -1
   fi
   p="$p$linkname"
#   echo "3: p = <$p>"
#   echo "${p:0:${#BASE}}"
   if [ "${p:0:${#BASE}}" != "$BASE" ]; then
      echo "can only modify data in $BASE"
      exit -1
   fi
#   echo "4: p = <$p>"
   find "$p" -xdev ! -type d -name \*.pdu-$DOTLESSVERSION \
      | grep '\.pdu-....'

   if [ $? -ne 0 ]; then
      echo "No inactive files found."
      return 0
   fi
   div
   local yn=$yn
   test -z $yn && read -p "Activate these file(s), old file(s) will be lost (y/n) [n]: " yn

   if [ "$yn" != "y" -a "$yn" != "j" ]; then
      return 0
   fi

   activated=0
   while IFS= read -r -d $'\0' f; do
#      echo "f = <$f>"
      b=${f%.pdu-$DOTLESSVERSION}
      bb=${b#$BASE}
#      echo "b = <$b>"
#      echo "bb = <$bb>"

      type=""
      if test -L "$f"; then
         type=LA
      elif test -d "$f"; then
         type=DA
      elif test -f "$f"; then
         type=FA
      fi

      if ! q=$(grep " $bb\$" $BASE/UPDATE.INF/data.inf); then
         echo "Skipped $f because $bb is not in the Server Environment."
      elif [ "${q:0:2}" = "$type" ]; then
         mv "$f" "$b"
         ((++activated))
      else
         echo "Skipped because $bb of type <${q:0:2}> is in the Server Environment but $f has different type <$type>"
      fi
   done < <(find "$p" -xdev ! -type d -name \*.pdu-$DOTLESSVERSION -print0)
   div
   echo "Activated $activated file(s)."
}

remove_old ()
{
   find ${BASE} -xdev -name \*.pdu-???? ! -name \*.pdu-$DOTLESSVERSION \
      ! -type d \
      | grep '\.pdu-....'

   if [ $? -ne 0 ]; then
      echo "no .pdu-???? files found"
      return 0
   fi

   local yn=$yn
   test -z $yn && read -p "delete these files (y/n) [n]: " yn
   echo "yn = <$yn>"

   if [ "$yn" = "y" -o "$yn" = "j" ]; then
      find ${BASE} -xdev -name \*.pdu-???? ! -name \*.pdu-$DOTLESSVERSION \
         ! -type d \
         -exec rm -f {} \;
   fi
}

show_inactive ()
{
   find ${BASE} -xdev -name \*.pdu-$DOTLESSVERSION \
      | grep '\.pdu-....'

   if [ $? -ne 0 ]; then
      echo "no 'inactive' pdu files found"
      return 0
   fi
}

remove_mysql_dirs ()
{
   find ${BASE}/var -xdev -type d -name mysql.????.????-??-??T??:??:?? \
      ! -name mysql.$DOTLESSVERSION.* \
      | grep 'mysql\.....\.'

   if [ $? -ne 0 ]; then
      echo "no old mysql template dirs found"
      return 0
   fi
   local yn=$yn
   test -z $yn && read -p "delete these directories (y/n) [n]: " yn
   echo "yn = <$yn>"

   if [ "$yn" = "y" -o "$yn" = "j" ]; then
      find ${BASE}/var -xdev \
         -depth \
         -type d \
         -name mysql.????.????-??-??T??:??:?? \
         ! -name mysql.$DOTLESSVERSION.* \
         -exec rm -fr {} \;
   fi
}

list_se_backups ()
{
   trap "$(shopt -p nullglob)" RETURN
   shopt -s nullglob
   se_backups=(/usr/local/pd-admin2.*)
   num=${#se_backups[@]}
   if [ $num -eq 0 ]; then
      echo "no SE backups found"
      return
   fi
   ls -laFd "${se_backups[@]}"
}

remove_se_backups ()
{
   trap "$(shopt -p nullglob)" RETURN
   shopt -s nullglob
   se_backups=(/usr/local/pd-admin2.*)
   num=${#se_backups[@]}
   if [ $num -eq 0 ]; then
      echo "no SE backups found"
      return
   elif [ $num -eq 1 ]; then
      echo "one SE backup found and kept"
      return
   fi
   let --num
   test -z $yn && read -p "delete these files (y/n) [n]: " yn
   test -z $yn && echo "dry run"
   for d in "${se_backups[@]:0:$num}"; do
      echo "removing <$d> ..."
      test -z $yn || rm -fr "$d"
   done
}

usage ()
{
   cat <<'_'
pduclean $Revision: 1.14 $
usage: pduclean.sh [-a path] [-M] [-m] [-r] [-s] [-y]

   -a path 
         activate 'inactive' pdu files in path

   -M    removes old mysql template files
         /usr/local/pd-admin2/var/mysql.<dotlessversion>.<timestamp>
         Keeps the templates having the current SE's version.

   -m    remove files with multiple pdu suffixes, keep latest and
         assign a single pdu suffix to it

   -r    remove files with suffix .pdu-<dotlessversion>
         where <dotlessversion> is different from the current SE's
         dotlessversion

   -s    show 'inactive' pdu files 

   -b    list SE backups

   -B    remove all SE backups up to the latest

   -y    yes to all questions
_
}

while getopts "a:bBhMmrsy" opt; do
   case $opt in
      a)
         echo "activate inactive pdu files <$OPTARG>"
         activate
         ;;
      b)
         echo "list SE backups"
         list_se_backups
         ;;
      B)
         echo "remove all SE backups up to the latest"
         remove_se_backups
         ;;
      M)
         echo "remove old mysql templates"
         remove_mysql_dirs
         ;;
      m)
         echo "remove files with multiple pdu suffixes"
         multi_suffix
         ;;
      r)
         echo "remove old"
         remove_old
         ;;
      s)
         echo "show 'inactive' pdu files"
         show_inactive
         ;;
      y)
         yn=y
         ;;
      h)
         usage
         exit
         ;;
   esac 
done
#echo "OPTIND = <$OPTIND>"
#echo "OPTARG = <$OPTARG>"
test $OPTIND = 1 && usage
