#!/bin/bash
#
#   se-update.sh (3)
#
#   update Server Environment (SE)
#
#   $Id: se-update.sh,v 1.172 2022/11/21 11:55:31 testx Exp testx $
#

SEUVERSION=2022112100

if [ $UID -ne 0 ]; then
   echo "Must be root to update the server environment!"
   exit 1
fi

if [ `umask` != '0022' ]; then 
   echo "Warning: umask is set to `umask`, should be 0022."
   echo "Setting umask to 0022"
   umask 0022
   sleep 3 
fi

   BASE=/usr/local
BASEDIR=$BASE/pd-admin2
   HERE=`pwd`
   PATH=$BASEDIR/bin:$PATH
  FORCE=0
  SETGZ=std-env-current.tar.gz

S64=
if $HERE/64 >/dev/null 2>&1; then
   S64=64
fi

export BASEDIR

function is_illegal_series_designator ()
{
   declare -a legal=(1 2 3 4 6 7 8 9)
   for sr in "${legal[@]}"; do
      if [ "$sr" = "$1" ]; then
         return 1
      fi
   done
   return 0
}

function chkfl ()
{
   if [ ! -f $1 ]; then
      echo "file $1"
      echo "not found, cannot update"
      exit 1
   fi
}
function chkdr ()
{
   if [ ! -d $1 ]; then
      echo "directory $1"
      echo "not found, cannot update"
      exit 1
   fi
}
function findfl ()
{
   VAR=$1
   CMD=$2
   T=`type -p $CMD 2>/dev/null`
   if [ $? -ne 0 ]; then
       echo "could not find $CMD"
       exit 8
    fi
    eval $VAR=$T
}

chkdr $BASEDIR
chkdr /service
chkfl /command/svc
chkfl ${HERE}/identify2.pl
chkfl ${HERE}/massunlink.pl
chkfl ${HERE}/chstructure.pl
chkfl ${HERE}/waitformysql.pl
chkfl ${HERE}/ipv6prep.sh
chkfl ${HERE}/pwhashupgrade.pl

findfl AWK awk
findfl SED sed
findfl DATA date
findfl MKTEMP mktemp
findfl DF df
findfl DU du
findfl CUT cut
findfl GREP grep
findfl MD5SUM md5sum
findfl CMP cmp
findfl TAIL tail
findfl FREESPACE ${HERE}/freespace$S64
findfl READLINK ${HERE}/readlink$S64
findfl MYCP ${HERE}/cp$S64
findfl T1523594 ${HERE}/ticket-1523594.sh
findfl T1524289 ${HERE}/ticket-1524289.sh

echo "FREESPACE = <$FREESPACE>"
echo "READLINK = <$READLINK>"
echo "MYCP = <$MYCP>"

FAULT=0
mycnfdef=${BASEDIR}/etc/my.cnf
for n in /etc/my.cnf /etc/mysql/my.cnf; do
   if [ -L $n ] || [ -f $n ]; then
      echo "Please remove $n"
      FAULT=1
   fi
done

if [ $FAULT -ne 0 ]; then
   echo "exiting ..."
   exit 2
fi

RQM=false
function rebindqmail ()
{
   #
   #  qmail einbinden
   #
   if [ -d /usr/local/pd-admin2/qmail/owners ]; then
      if [ ! -L /var/qmail/owners -a ! -d /var/qmail/owners ]; then
         echo '** qmail/owners wird verlinkt **'
         ln -s $BASEDIR/qmail/owners /var/qmail
      fi
      if [ ! -L /var/qmail/bin -a -d /var/qmail/bin ]; then
         echo '** qmail/bin wird verlinkt **'
         mv /var/qmail/bin /var/qmail/bin.$DS
         ln -s $BASEDIR/qmail/bin /var/qmail/bin
         RQM=true
      fi
   fi

   #
   #  fix perms
   #
   chown qmaild.qmail /var/qmail/control
   chmod 750 /var/qmail/control

   chgrp qmail \
/var/qmail/bin/qmail-start \
/var/qmail/bin/qmail-getpw \
/var/qmail/bin/qmail-local \
/var/qmail/bin/splogger \
/var/qmail/bin/qmail-lspawn \
/var/qmail/bin/qmail-rspawn \
/var/qmail/bin/qmail-clean \
/var/qmail/bin/qmail-newu \
/var/qmail/bin/qmail-newmrh \
/var/qmail/bin/qmail-pw2u \
/var/qmail/bin/qmail-inject \
/var/qmail/bin/predate \
/var/qmail/bin/datemail \
/var/qmail/bin/mailsubj \
/var/qmail/bin/qmail-showctl \
/var/qmail/bin/qmail-qread \
/var/qmail/bin/qmail-qstat \
/var/qmail/bin/qmail-tcpto \
/var/qmail/bin/qmail-tcpok \
/var/qmail/bin/qmail-pop3d \
/var/qmail/bin/qmail-send \
/var/qmail/bin/qmail-qmqpc \
/var/qmail/bin/qmail-qmqpd \
/var/qmail/bin/qmail-qmtpd \
/var/qmail/bin/qmail-remote \
/var/qmail/bin/sendmail \
/var/qmail/bin/tcp-env \
/var/qmail/bin/qreceipt \
/var/qmail/bin/qsmhook \
/var/qmail/bin/qbiff \
/var/qmail/bin/forward \
/var/qmail/bin/preline \
/var/qmail/bin/condredirect \
/var/qmail/bin/bouncesaying \
/var/qmail/bin/except \
/var/qmail/bin/maildirmake \
/var/qmail/bin/maildir2mbox \
/var/qmail/bin/maildirwatch \
/var/qmail/bin/qail \
/var/qmail/bin/elq \
/var/qmail/bin/pinq \
/var/qmail/bin/qmail-popup \
/var/qmail/bin/qmail-smtpd

   chmod 755 /var/qmail/bin/qmail-inject
   chmod g+s /var/qmail/bin/qmail-inject
   chmod 755 /var/qmail/bin/sendmail
   chmod g+s /var/qmail/bin/sendmail

   touch /var/qmail/control/smtpplugins
   touch /var/qmail/control/taps
}

function usage ()
{
   echo " "
   echo "se-update.sh [ -s Reihe | -f se-file.tgz ]"
   echo " "
   echo "  -s Reihe       : Upgrade auf Reihe erzwingen"
   echo "  -f se-file.tgz : Update auf se-file.tgz erzwingen"
   echo " "
}

OPTS=
OPTF=
DOVECOT_MIGRATION=1
while getopts 'cs:f:x:h?' X; do
   case $X in
      c ) DOVECOT_MIGRATION=0;;
      s ) OPTS=$OPTARG; FORCE=1; NEWSERIES=$OPTARG;;
      f ) OPTF=$OPTARG; FORCE=1; NEWSETGZ=$OPTARG;;
      x ) OPTX=$OPTARG;; 
      h | ? ) usage; exit 0 ;;
      * ) usage; exit 1;;
   esac
done
#echo "OPTS=$OPTS, OPTF=$OPTF"

if [ "$OPTS" != "" -a "$OPTF" != "" ]; then
   usage
   exit 2
fi

function mktempcheck ()
{
#   S=`$MKTEMP -d -p/tmp iiiXXXXXX 2>/dev/null`
   S=`$MKTEMP -d /tmp/iiiXXXXXX 2>/dev/null`
   if [ $? -ne 0 ]; then 
      echo "Your mktemp command"
      echo " "
      echo "    " ${MKTEMP}
      echo " "
      echo "is not capable of creating directories. Please update"
      echo "your OS utilities!"
      exit 56
   fi
   rmdir $S
}
mktempcheck

function check_space_requirements ()
{
   FREE=`${FREESPACE} ${BASE}`
   if [ $? -ne 0 ]; then
      echo "freespace command failed"
      exit 57
   fi
   OCCU=`$DU -ms $BASEDIR | $CUT -f1`
   if [ $? -ne 0 ]; then
      echo "du/cut command failed"
      exit 58
   fi

#
#  add 10 percent for se's growth
#
   NEED=$((OCCU * 11 / 10))
   printf "Plattenplatz belegt  : %6d MiB\n" $OCCU
   printf "             bentigt: %6d MiB (geschtzt)\n" $NEED
   printf "             frei    : %6d MiB\n" $FREE
   if [ $FREE -lt $NEED ]; then
      echo "|"
      echo "| Der freie Festplattenplatz reicht mglicherweise nicht"
      echo "| aus, um ein Update erfolgreich durchzufhren. Es mssen"
      echo "| mindestens $OCCU MiB freier Plattenplatz verfgbar sein,"
      echo "| zurzeit sind aber nur $FREE MiB verfgbar."
      echo "|"
      exit 999
   fi
   echo " "
}
function start_services ()
{
   echo "|"
   echo "| Dienste werden gestartet ..."
   echo "|"
   /command/svc -u /service/mysqld
   sleep 6
   for i in /service/*{,/log}; do
      echo  $i
      /command/svc -u $i
   done
   sleep 3
}
function start_services2 ()
{
   if [ ! -e /service/apache -a ! -e /service/apache24 ]; then
      echo "Apache wird gestartet"
      ${BASEDIR}/bin/apachectl start
      if [ $? -ne 0 ]; then
        echo "Der Apache konnte nicht erfolgreich ausgefhrt werden."
      fi
   fi
}
function start_services3 ()
{
   if [ -f $BASEDIR/sbin/authdaemond ]; then
      if [ $AUTHD -eq 0 -o $AUTHDRUN -eq 0 ]; then
         echo "starte authdaemond ..."
         $BASEDIR/sbin/authdaemond start
      fi
   fi

   if [ $IMAPD -eq 0 ]; then
      echo "starte imapd ..."
      $BASEDIR/libexec/imapd.rc start
   fi

   if [ $IMAPDSSL -eq 0 ]; then
      echo "starte imapd-ssl ..."
      $BASEDIR/libexec/imapd-ssl.rc start
   fi
}

function remove_setemp ()
{
   echo "entpackte SE in $SETEMP wird entfernt"
   if [ -d $SETEMP ]; then
      rm -fr $SETEMP
   fi
}

function remove_basedir_work ()
{
   if [ -d $BASEDIR.work ]; then
      echo "Arbeitsverzeichnis $BASEDIR.work wird entfernt ..."
      rm -fr $BASEDIR.work
   fi
}

function rollback ()
{
   if [ "$DS" == "" ]; then
      echo "STOP: DS=''"
      exit 10
   fi
   if [ "$BASEDIR" == "" ]; then
      echo "STOP: BASEDIR=''"
      exit 20
   fi
   if [ ! -d $BASEDIR ]; then
      echo "$BASEDIR existiert nicht. Widerherstellung nicht mglich"
      exit 30
   fi
   if [ ! -d $BASEDIR.$DS ]; then
      echo "$BASEDIR.$DS existiert nicht. Widerherstellung nicht mglich"
      exit 40
   fi

   echo " "
   echo "Wiederherstellung des Zustands vor Beginn des Updates luft ..."
   echo " "

   if $RQM; then
      echo 'RQM: altes qmail-Verzeichnis wird wiederhergestellt'
      rm /var/qmail/bin
      mv /var/qmail/bin.$DS /var/qmail/bin
   fi

   if [ -d $BASEDIR.work ]; then
      echo "Arbeitsverzeichnis $BASEDIR.work wird entfernt ..."
      rm -fr $BASEDIR.work
   else
      echo "Arbeitsverzeichnis $BASEDIR.work nicht gefunden."
      exit 2
   fi

   if [ -d $BASEDIR.$DS ]; then
      echo "Sicherungsverzeichnis $BASEDIR.$DS gefunden."
      echo "Verzeichnis $BASEDIR wird entfernt ..."
      rm -fr $BASEDIR 
      if [ $? -ne 0 ]; then
         echo "Fehler: Verzeichnis konnte nicht entfernt werden"
         exit 3
      fi
      echo "Sicherungsverzeichnis $BASEDIR.$DS wird in $BASEDIR umbenannt"
      mv $BASEDIR.$DS $BASEDIR
      if [ $? -ne 0 ]; then
         echo "Fehler: $BASEDIR.$DS konnt nicht umbenannt werden."
         exit 4
      fi
   fi

   echo " "
   echo "Wiederanfahren der bekannten Dienste (/services/* und"
   echo "apache-Webserver). Bitte prfen Sie, ob die Dienste"
   echo "wieder vollstndig angelaufen sind."
   echo " "

   start_services
   start_services2
   start_services3
}

#
#
#

function correct_httpd24_LogFormat ()
# Ticket 1237233
{
   local tmpl=$BASEDIR/httpd-2.4/conf/httpd24.conf-template
   if ! grep -q '%v:' $tmpl; then
      return
   fi

   echo "| "
   echo "| LogFormat in $tmpl anpassen"
   echo "| "

   $SED -i 's,LogFormat.* vcombined,LogFormat "%{Host}i %h %l %<u %t \\"%r\\" %s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\" " vcombined,' \
      $tmpl
}

function performe ()
{
echo "| "
echo "| Update der Serverumgebung (SE) (seu3)"
echo '| $Id: se-update.sh,v 1.172 2022/11/21 11:55:31 testx Exp testx $ '
echo "| "
echo " "

check_space_requirements

#
#  detect SERIES
#
if [ ! -f $BASEDIR/UPDATE.INF/SERIES ]; then
   echo "| "
   echo "| Die installierte Reihe (SERIES) der SE konnte nicht bestimmt werden"
   echo "| Die installierte Version der SE ist zu alt und kann mit "
   echo "| diesem se-update.sh nicht geupdatet werden."
   echo "| Bitte kontaktieren Sie providerdienste.de"
   echo "| "
   exit 451
else
   SERIES=`cat $BASEDIR/UPDATE.INF/SERIES`
fi
echo "| "
echo "| installierte SE-Reihe : $SERIES"
echo "| "

if [ "$NEWSERIES" != "" ]; then
   if is_illegal_series_designator "$NEWSERIES"; then
      echo "NEWSERIES <$NEWSERIES> is not a valid series designator"
      exit 84
   fi
   if [ "$NEWSERIES" -lt "$SERIES" ]; then
      echo "cannot downgrade from series <$SERIES> to series <$NEWSERIES>"
      exit 85
   fi
   if [ "$SERIES" -eq "6" -a "$NEWSERIES" -eq "8" ]; then
      echo "upgrading from series 6 to 8"
   elif [ "$NEWSERIES" -ge "6" ]; then
      echo "cannot upgrade <$SERIES> to series <$NEWSERIES> (unsupported upgrade path)"
      exit 87
   fi
   echo "| "
   echo "| Upgrade von Reihe $SERIES auf $NEWSERIES wird durchgefhrt"
   echo "| "
   SERIES=$NEWSERIES
fi

#
#  Ticket 1523369
#
if [ "$S64" = "" ]; then
   case $SERIES in
      1 | 2 | 3) # 32 bit
         ;;
      *)         # new SE's are all 64 bit
         echo "cannot update/upgrade to Series $SERIES"
         echo "the operating system does not execute 64-Bit-Binaries"
         exit 64
   esac
fi

#
#  fetch latest OL (on-line) VERSION from server
#
rm -f SE-LATEST
wget http://download.pd-admin.de/SE-LATEST
if [ $? -ne 0 ]; then
   echo "Datei SE-LATEST konnte nicht vom Server geholt werden."
   exit 4566
fi
OLVERSION=`cut -c1,3-5 SE-LATEST`

if [ ! -f $BASEDIR/UPDATE.INF/data.inf ]; then
   echo "The installed Server Environment (SE) lacks a file named UPDATE.INF/data.inf"
   echo "This file is present since SE 0.014 of April 2004"
   echo "Sorry, cannot update your installation"
   exit 1
fi

chkdr $BASEDIR/UPDATE.INF
chkfl $BASEDIR/UPDATE.INF/data.inf

#
#  detect VERSION
#
if [ ! -f $BASEDIR/UPDATE.INF/VERSION ]; then
   INSTVERSION=`head -1 $BASEDIR/UPDATE.INF/data.inf \
      | cut -f3 -d\  | cut -c1,3-5`
else
   INSTVERSION=`cat $BASEDIR/UPDATE.INF/VERSION \
      | cut -c1,3-5`
fi

echo "| "
echo "|     installierte Version: $INSTVERSION"
echo "| auf dem Server verfgbar: $OLVERSION"
echo "| "

if [[ 10#$INSTVERSION < 10#0099 ]]; then
   echo "Installed Version <$INSTVERSION> too old for this update script"
   echo " need at least Version 0.099 of 2007 (first version with Apache 2)"
   exit 2
fi

SETGZ="se-$SERIES-`cat SE-LATEST`.tar.gz"
if [ $FORCE -eq 0 ]; then
   if [ $INSTVERSION -ge $OLVERSION ]; then
      echo "Es ist keine neuere Version der Serverumgebung verfgbar."
      exit 0
   fi
else
   if [ "$NEWSETGZ" != "" ]; then
      SETGZ=$NEWSETGZ
   fi
fi


export DS=`date +%Y-%m-%dT%H:%M:%S 2>/dev/null`
if [ $? -ne 0 ]; then
   echo "date function error"
   exit 2
fi
#echo $DS

RECENT=0014

function stop_apache ()
{
   echo "|"
   echo "| trying to stop apache ..."
   echo "|"
   if [ -e /service/apache ]; then
      svc -d /service/apache
      $BASEDIR/bin/apachectl stop 2>/dev/null 1>/dev/null
   fi
   if [ -e /service/apache24 ]; then
      svc -d /service/apache24
      $BASEDIR/httpd-2.4/bin/apachectl stop 2>/dev/null 1>/dev/null
   fi
   sleep 1
   killall -9 httpd
}
function start_apache ()
{
   echo "|"
   echo "| trying to start apache ..."
   echo "|"
   if [ -e /service/apache ]; then
      svc -u /service/apache
   elif [ -e /service/apache24 ]; then
      svc -u /service/apache24
   else
      echo "** Apache konnte nicht gestartet werden, /service/apache[24]"
      echo "** fehlt."
   fi
}

rm -f VERSION OLDFILES CHANGEDFILES NEWFILES  
echo "Versions- und Dateiberprfung wird gestartet"
./identify2.pl

echo "Lade die Server-Umgebung (SE) herunter (ca. 350 MB)"
wget -nc http://download.pd-admin.de/SE-SERIES/${SETGZ}
if [ $? -ne 0 ]; then
   echo "download of server environment failed"
   exit 8
fi

if [ -f ${SETGZ} ]; then
   echo "Prfe tar-File"
   CVERSION=`tar tfz ${SETGZ} | head -1 | cut -c2,4-6`
   MARKER=`tar tfz ${SETGZ} | head -1 | cut -c1,7`

   if [ "$MARKER" != "[]" ]; then
      echo "tar label konnte nicht identifiziert werden."
      exit 9
   fi

   echo "found $CVERSION in tar file"
   RECENT=$CVERSION
fi

if [ -f VERSION ]; then
   VERSION=`cat VERSION`
   if [ $VERSION -eq $RECENT ]; then
      if [ $FORCE -ne 1 ]; then
         echo ""
         echo "Die aktuelle Version $VERSION ist bereits installiert."
         exit 0
      else
         echo ""
         echo "Die aktuelle Version $VERSION ist bereits installiert,"
         echo "Update wird dennoch durchgefhrt (-f)"
      fi
   elif [ $VERSION -gt $RECENT ]; then
      echo ""
      echo "Die Versionsnummer $VERSION der installierten Version ist"
      echo "grer als die Versionnummer der aktuellen Version."
      echo ""
      exit 0
   else
      echo ""
      echo "Installierte Version: $VERSION"
      echo "Aktuelle Version    : $RECENT"
      echo "Die Installation der aktuellen Version wird NUN vorbereitet ..."
      echo "" 
   fi
else
   echo " Fehler"
   echo " Die installierte Version konnte nicht identifiziert werden."
   exit 5
fi

#
#  2004-11-23 stvo
#  unpack SE into temp directory
#
SETEMP=`$MKTEMP -d $BASEDIR.XXXXXX`
if [ $? -ne 0 ]; then
   echo "could not create temporary directory"
   exit 20
fi
echo "SETEMP=$SETEMP"
pushd $SETEMP 1>/dev/null 2>&1
echo "Entpacke Server-Umgebung. Bitte haben Sie etwas Geduld."
sleep 3
tar kxfz $HERE/${SETGZ} || {
   echo "Kann ${SETGZ} nicht entpacken"
   exit 10
}
popd

#  2008-01-31

if [ -f ${SETEMP}/pd-admin2/UPDATE.INF/MINSEU ]; then
   read MINSEU <${SETEMP}/pd-admin2/UPDATE.INF/MINSEU
   if [ $SEUVERSION -ge $MINSEU ]; then
      echo "Versionsvergleich $SEUVERSION >= $MINSEU OK."
   else
      echo ""
      echo "Versionsnummer se-update.sh: $SEUVERSION"
      echo "erforderlich fr neue SE   : $MINSEU"
      echo "Bitte laden Sie das aktuelle Update-Paket herunter:"
      echo ""
      echo "   wget http://download.pd-admin.de/seu3"
      echo ""
      echo "(Die entpackte SE in $SETEMP wird entfernt.)"
      rm -fr ${SETEMP}
      exit 11
   fi
else
   echo ""
   echo "Die heruntergeladene Serverumgebung kann mit diesem"
   echo "Skript nicht verwendet werden."
   echo "(MINSEU fehlt)"
   echo ""
   echo "(Die entpackte SE in $SETEMP wird entfernt.)"
   rm -fr ${SETEMP}
   exit 12
fi

#
#  2014-04-02 stvo
#  SE 0.235 swaps positions of postgresql 8 and 9
#  prevent update if postgresql is running
#
if [[ 10#$RECENT -ge 10#0235 && 10#$VERSION -le 10#0234 ]]; then
   if ps aux | grep [p]ostgres > /dev/null; then
      echo $"A running instance of PostgreSQL was detected."
      echo $"Please stop this instance before starting the SE update."
      echo $"Since SE 0.235 PostgreSQL 9.0 has become the main version"
      echo $"while before 0.235 the main version was 8.3.23."
      echo $"Please adjust your PostgreSQL startup script in order to"
      echo $"use the correct version's PostgreSQL executable."
      echo $"(The unpacked SE in $SETEMP will be removed now.)"
      rm -fr ${SETEMP}
      exit 13
   fi
fi

#
#  Reihenwechsel erkennen
#
SESO=`cat $BASEDIR/UPDATE.INF/SERIES`
SESN=`cat $SETEMP/pd-admin2/UPDATE.INF/SERIES`
if [ $SESO -ne $SESN ]; then
   SESC=$SESO$SESN
fi

if [ "$SESN" == "5" -a "$SESO" != "5" -a "$OPTX" != "upgrade" ]; then
   echo "| "
   echo "| Upgrade auf Reihe 5 ist nicht mglich."
   echo "| "
   remove_setemp
   exit 8 
fi

#-------------------------->
#  time critical begin
#-------------------------->

stop_apache

MSRPW2=/etc/admin/mysql_rootpw
MSRPW3=/opt/pdadmin/etc/mysql_rootpw.conf
MSRPW4=/opt/pdadmin/etc/mysql_rootpw.conf_local

if [ -f $MSRPW2 ]; then
   MSRPW=$MSRPW2
fi
if [ -f $MSRPW3 ]; then
   MSRPW=$MSRPW3
fi
if [ -f $MSRPW4 ]; then
   MSRPW=$MSRPW4
fi

if [ "$BACKUPPATH" != "" ]; then
   echo "| "
   echo "| Backup der MySQL-Datenbanken"
   echo "| Ziel: $BACKUPPATH "
   echo "| "

   if [ -f $BACKUPPATH ]; then
      echo "Datei $BACKUPPATH existiert bereits. Sie wird in"
      BNEW=`$MKTEMP $BACKUPPATH.XXXXXX`
      echo "$BNEW umbenannt."
      mv $BACKUPPATH $BNEW
      if [ $? -ne 0 ]; then
         echo "** Fehler: Konnte Backup nicht umbenennen."
         start_apache
         remove_setemp
         exit 76
      fi
   elif [ -e $BACKUPPATH ]; then
      echo "** Fehler: $BACKUPPATH existiert ist aber keine Datei."
         start_apache
         remove_setemp
         exit 77
   fi

   $BASEDIR/bin/mysqldump -FQaAp`cat $MSRPW` \
      > $BACKUPPATH
   if [ $? -ne 0 ]; then
      echo "** Fehler: Konnte Backup nicht nach $BACKUPPATH schreiben."
      start_apache
      remove_setemp
      exit 78
   fi
fi

echo "| "
echo "| Dienste werden gestoppt ..."
echo "| "
for i in /service/*{,/log}; do
   echo  $i
   /command/svc -d $i
done
#
#     tcpserver from qmail-smtpd
#
killall -s TERM tcpserver

#
#    courier imap
#
if [ -d /service/courier-authd ]; then
   AUTHD=7
   AUTHDRUN=$AUTHD
else
   ps aux|grep [a]uthdaemond 2>&1 > /dev/null
   AUTHD=$?
   AUTHDRUN=$AUTHD
fi


if [ -d /service/courier-imap ]; then
   IMAPD=7
else
   ps aux|grep "[i]mapd " 2>&1 > /dev/null
   IMAPD=$?
   if [ $IMAPD -eq 0 ]; then
      echo "stoppe imapd ..."
      $BASEDIR/libexec/imapd.rc stop
      AUTHD=0
   fi
fi

if [ -d /service/courier-imap-ssl ]; then
   IMAPDSSL=7
else
   ps aux|grep [i]mapd-ssl 2>&1 > /dev/null
   IMAPDSSL=$?
   if [ $IMAPDSSL -eq 0 ]; then
      echo "stoppe imapd-ssl ..."
      $BASEDIR/libexec/imapd-ssl.rc stop
      AUTHD=0
   fi
fi

if [ $AUTHDRUN -eq 0 -a -f $BASEDIR/sbin/authdaemond ]; then
   echo "stoppe authdaemond ..."
   $BASEDIR/sbin/authdaemond stop
fi

killall -s TERM couriertls
killall -s TERM courierlogger

#
#     mysql 
#
function mysql_shutdown ()
{
   svc -d /service/mysqld
   local try_count=75

   while killall -0 mysqld; do
      if [[ $try_count = 0 ]]; then
         false
         return
      fi      
      echo "mysql still running ($try_count) `date`"
      killall mysqld
      sleep 4 
      ((--try_count))
   done
   true
}

if ! mysql_shutdown; then
   echo " fail"
   echo "** Fehler: Konnte den MySQL-Server nicht stoppen"
   echo "** Systemaktualisierung wird abgebrochen, Dienste werden"
   echo "** werden wieder gestartet."
   start_services
   start_services2
   start_services3
   remove_setemp
   exit 2  
fi

echo -n "Lege ein temporres Arbeitsverzeichnis an: $BASEDIR.work"
if [ -d $BASEDIR.work ]; then
   echo " "
   echo "   $BASEDIR.work bereits vorhanden. Verzeichnis wird nach"
   BWB=`$MKTEMP -d $BASEDIR.work.XXXXXX`
   echo "   $BWB verschoben."
   mv $BASEDIR.work $BWB/
   if [ $? -ne 0 ]; then
      echo " Fehler: "
      echo "Konnte nicht verschieben."
      exit 3
   else 
      echo "   done."
      echo -n "Lege ein temporres Arbeitsverzeichnis an: $BASEDIR.work"
   fi
fi
mkdir $BASEDIR.work
if [ $? -ne 0 ]; then
   echo " Fehler: "
   echo "Konnte Verzeichnis $BASEDIR.work nicht anlegen."
   exit 3
else
   echo " done"
fi

echo -n "Installierte Dateien werden kopiert (hard-linking) ..."
cp -axl $BASEDIR/. $BASEDIR.work
if [ $? -ne 0 ]; then
   echo " Fehler"
   echo "Konnte Dateien nicht kopieren (hard linken)."
   start_services
   start_services2
   start_services3
   echo "done"
   echo "temporres Verzeichnis $BASEDIR.work wird entfernt"
   rm -fr $BASEDIR.work

   remove_setemp

   rm -fr $BASEDIR.work
   exit 4
else
   echo " done"
fi

echo "Unvernderte Dateien werden gelscht."
rm -f $BASEDIR.work/UPDATE.INF/data{,2}.inf
rm -f $BASEDIR.work/UPDATE.INF/hotfix-*.inf
$HERE/massunlink.pl OLDFILES
if [ $? -ne 0 ]; then
   echo "|"
   echo "| massunlink ist fehlgeschlagen"
   echo "|"
   remove_setemp
   remove_basedir_work
   start_services
   start_services2
   start_services3
   exit 4
fi
rm -f $BASEDIR.work/UPDATE.INF/SERIES
rm -f $BASEDIR.work/UPDATE.INF/VERSION
rm -f $BASEDIR.work/UPDATE.INF/MINSEU
if [ $? -ne 0 ]; then
   echo " Fehler"
   echo "Konnte die Dateilschung nicht abschlieen."
   exit 5
else
   echo " done"
fi
rm -f $BASEDIR.work/SETENV
rm -f $BASEDIR.work/post-install.sh

if [ $INSTVERSION -lt 0325 -a $CVERSION -ge 0325 ]; then
#
#  Ticket 1523356: SE: phpMyAdmin security
#
   echo "|"
   echo "| removing old phpMyAdmin and phpPgAdmin versions"
   echo "|"
   rm -fr ${BASEDIR}.work/htdocs/phpMyAdmin{,2,46}
   rm -fr ${BASEDIR}.work/htdocs/phpPgAdmin
fi

function unlink_symlinks_to_crs ()
{
   local src=$1
   shift
   local dst=$1
   shift
   echo "src <$src> dst <$dst>"
   if local dirname=$($READLINK $src) && [[ $dirname = $dst ]]
   then
      echo "unlinking src <$src>"
      unlink $src
   else
      echo "dirname <$dirname>"
   fi
}

unlink_symlinks_to_crs $BASEDIR.work/conf/coreruleset-current $BASEDIR/conf/coreruleset-3.3.0
unlink_symlinks_to_crs $BASEDIR.work/httpd-2.4/conf/coreruleset-current $BASEDIR/httpd-2.4/conf/coreruleset-3.3.0

echo -n "Das alte pd-admin2-Verzeichnis wird umbenannt ..."
if [ -L $BASEDIR.$DS ] || [ -d $BASEDIR.$DS ] \
   || [ -f $BASEDIR.$DS ]; then
   echo " Fehler:"
   echo "Ein symbolischer Link/ein Verzeichnis mit diesem Namen ist bereits vorhanden."
   exit 6
fi

mv $BASEDIR $BASEDIR.$DS
if [ $? -ne 0 ]; then
   echo " Fehler:"
   echo "Konnte Das Verzeichnis nicht umbenennen."
   exit 7
else
   echo " done"
fi


pushd $BASE 1>/dev/null 2>&1
echo "Bringe die neue Server-Umgebung in Stellung..."
mv --target-directory=$BASE $SETEMP/pd-admin2
if [ $? -ne 0 ]; then
   echo "could not rename $SETEMP to $BASEDIR"
   exit 10
fi

rmdir $SETEMP
if [ $? -ne 0 ]; then
   echo "could not rmdir $SETEMP"
   exit 22
fi

popd
$BASEDIR/post-install.sh

rebindqmail

echo "HERE=$HERE"

test -L $BASEDIR.work/var/mysql && {
   echo "Entferne Symlink $BASEDIR.work/var/mysql"
   MYSQLDATADIR=`$READLINK $BASEDIR.work/var/mysql`
   echo "(1) MYSQLDATADIR = <$MYSQLDATADIR>"
   rm $BASEDIR.work/var/mysql
}

echo "checking subdirectory structure before importing changed files"
$HERE/chstructure.pl $BASEDIR $BASEDIR.work
if [ $? -ne 0 ]; then
   echo " "
   echo "*** Import der neuen/genderten Dateien/Verzeichnisse ***"
   echo "*** kann wegen eines Dateitypkonflikts nicht vorge-   ***"
   echo "*** nommen werden.                                    ***"
   echo " "
   rollback
   exit 12
fi

echo "importing changed files from work dir"

$MYCP -ax --backup=simple --suffix=.pdu-$RECENT \
   $BASEDIR.work/. \
   $BASEDIR
if [ $? -ne 0 ]; then
   echo "import failed"
   rollback
   exit 11
fi

rm ./pd-admin2

echo -n "Arbeitsverzeichnis wird gelscht ..."
rm -fr $BASEDIR.work
echo "Fertig"


echo "| "
echo "| Entferne einige .pdu-Dateien ..."
echo "| "

for i in ${BASEDIR}\
/var/mysql/mysql/\
{db,user,tables_priv,func,host,columns_priv,time_zone\
{,_leap_seconds,_name,_transition{,_type}}}.{MY[ID],frm}.pdu-*; do
   echo "$i"
   rm -f "$i"
done

echo "(2) MYSQLDATADIR = <$MYSQLDATADIR>"
if [ ! "$MYSQLDATADIR" == "" ]; then
   for i in $BASEDIR/var/mysql/mysql/*; do
      B=`basename $i`
      if [ ! -e $MYSQLDATADIR/mysql/$B ]; then
         mv $i $MYSQLDATADIR/mysql/
      fi
   done
   mv $BASEDIR/var/mysql $BASEDIR/var/mysql.$RECENT.$DS
   chown -hR mysql:daemon $MYSQLDATADIR
   echo "(3) re-creating symlink var/mysql -> <$MYSQLDATADIR>"
   ln -s $MYSQLDATADIR $BASEDIR/var/mysql
   ln -snf /usr/local/pd-admin2/etc/my.cnf \
      /usr/local/pd-admin2/var/mysql/
fi

#
#  2014-04-07 stvo
#  second check for mysql symlink
#
if [ -L "$BASEDIR.$DS/var/mysql" ]; then
   echo "old SE has mysql symlink"
   if [ -L "$BASEDIR/var/mysql" ]; then
      echo "new SE has mysql symlink (OK)"
   else
      echo "new SE has no mysql symlink (FAIL), trying to correct ..."
      mv "$BASEDIR/var/mysql" "$BASEDIR/var/mysql.$DS.$$"
      echo "move returned $?"
      cp -ax "$BASEDIR.$DS/var/mysql" "$BASEDIR/var/"
      echo "cp returned $?"
      if [ -L "$BASEDIR/var/mysql" ]; then
         echo "new SE has mysql symlink (OK)"
      else
         echo "new SE has no mysql symlink (FAIL), giving up"
         exit 19
      fi
   fi
fi

#  2006-01-17 stvo
mkdir -p /home/simscan/.spamassassin
chown spamd /home/simscan/.spamassassin
chmod 750 /home/simscan/.spamassassin

#
#  SE > 0.086: chown squirrel
#
if grep ${BASEDIR}/htdocs/squirrelmail /etc/passwd >/dev/null ; then
   echo "| "
   echo "| changing u/g of squirrelmail"
   echo "| "
   chown -hR squirrel:squirrel ${BASEDIR}/htdocs/squirrelmail
   grep webmail /etc/passwd >/dev/null && \
      find /tmp/ -type f -name sess_\* -user webmail -exec rm {} \;
fi


#
#  2011-08-01 stvo
#  check/recompile for IPv6
#
${HERE}/ipv6prep.sh

#
#  2012-07-31 stvo
#  prevent creation of /root/.razor
#
spamdrunfile=/service/spamd/run
socketindicator=$BASEDIR/etc/mail/spamassassin/spamc.conf
if test -f $socketindicator && grep -q spamd.sock $socketindicator \
   && ! grep -q spamd.sock $spamdrunfile ; then
   newrunfile=`mktemp /tmp/run.XXXXXXXXX`
   chmod +x $newrunfile
   cat > $newrunfile <<'_'
#!/bin/bash
PATH=/usr/local/pd-admin2/bin:/usr/local/pd-admin2/sbin:$PATH
exec setuidgid spamd /usr/local/pd-admin2/bin/spamd \
   --socketpath=/var/spamd/spamd.sock \
   --socketmode=0666
_
   cat >> $newrunfile <<_
#
# old run file before `date`
#
_
   while read line; do
      echo '# '$line >> $newrunfile
   done < $spamdrunfile
   mv $newrunfile $spamdrunfile
fi

#
#  2013-06-21 stvo
#  remove skip-locking and skip-federated from my.cnf
#  in case of SE-2 to -3 upgrade (MySQL 5.0 -> 5.1)
#
if [ "$SESC" -a "$SESN" == "3" -o "$SESN" == "4" -o "$SESN" == "5" ]; then
   $SED -e "s/^ *skip-locking/#skip-locking/g;" \
      -e "s/^ *skip-federated/#skip-federated/g" $BASEDIR/etc/my.cnf \
      > $BASEDIR/etc/my.cnf.new.$$ \
      && \
      $CMP -s $BASEDIR/etc/my.cnf $BASEDIR/etc/my.cnf.new.$$ \
      || (\
      mv $BASEDIR/etc/my.cnf.new.$$ $BASEDIR/etc/my.cnf \
      && echo "modified my.cnf" )
   rm -f $BASEDIR/etc/my.cnf.new.$$
fi

start_services
correct_httpd24_LogFormat

echo "| "
echo "| PHP-Version von 4 auf 5 aendern"
echo "| "

sed "s/phpwrap/php5wrap/" ${BASEDIR}/conf/httpd.conf-template \
| sed "s/php-cgiwrap/php5-cgiwrap/" \
> ${BASEDIR}/conf/httpd.conf-template_pdu0132
mv ${BASEDIR}/conf/httpd.conf-template_pdu0132 \
   ${BASEDIR}/conf/httpd.conf-template
${HERE}/waitformysql.pl
/opt/pdadmin/bin/httpd_vhosts.pl

#
#  2006-01-17
#  2007-04-18
#  fix_privilege tables after upgrade to mysql 4.1/5.0
#
echo "| "
echo "| MySQL fix privileges/MySQL shutdown/restart (MySQL 4.1/5.0)"
echo "| mysql_upgrade (MySQL 5.1)"
echo "| "

SKIPPED=1
$GREP '"4.1' ${BASEDIR}/include/mysql/mysql_version.h >/dev/null 2>&1 
if [ $? -eq 0 ]; then
   HOME=/dev/null $BASEDIR/bin/mysql_fix_privilege_tables \
      --password=`cat $MSRPW`
   SKIPPED=0
fi
$GREP '"5.0' ${BASEDIR}/include/mysql/mysql_version.h >/dev/null 2>&1 
if [ $? -eq 0 ]; then
   HOME=/dev/null $BASEDIR/bin/mysql_fix_privilege_tables \
      --password=`cat $MSRPW` \
      --basedir=$BASEDIR

   SKIPPED=0
fi
$GREP '"5.1' ${BASEDIR}/include/mysql/mysql_version.h >/dev/null 2>&1 \
   || $GREP '"5.5' ${BASEDIR}/include/mysql/mysql_version.h >/dev/null 2>&1 \
   || $GREP '"5.7' ${BASEDIR}/include/mysql/mysql_version.h >/dev/null 2>&1 
if [ $? -eq 0 ]; then
   echo "| "
   echo "| mysql_upgrade (5.1, 5.5, 5.7)"
   echo "| "
   sleep 3
   HOME=/dev/null $BASEDIR/bin/mysql_upgrade --password=$(cat $MSRPW)
   SKIPPED=0
fi
if [ $SKIPPED -eq 1 ]; then
   echo "skipping mysql_fix_privilege_tables (MySQL 4.0/5.0)"
   echo "skipping mysql_upgrade (MySQL 5.1)"
fi
#
#  nur beim Wechsel der Reihe
#
if [ $SESC ]; then
   echo "| "
   echo "| mysqlcheck --all-databases --repair"
   echo "| "
   sleep 3
   HOME=/dev/null $BASEDIR/bin/mysqlcheck --all-databases --repair \
      --password=$(cat $MSRPW)
fi

mysql_shutdown
svc -u /service/mysqld

chmod 750 $BASEDIR/conf

#
#  2004-11-09 stvo
#  make httpd.conf readable if Frontpage is installed
#
PDAC2=/etc/admin/config.pl
PDAC3=/opt/pdadmin/etc/pdadmin.conf

if [ -f $PDAC2 ]; then
   PDAC=$PDAC2
fi
if [ -f $PDAC3 ]; then
   PDAC=$PDAC3
fi

if [ -f $PDAC ]; then
   if grep -q '^[ ]*$use_frontpage[ ]*=[ ]*1' $PDAC; then
      chmod o+x $BASEDIR/conf
   fi
fi

start_services2

echo "| "
echo "| Courier IMAP wird initialisiert."
echo "| "

CONFCI=1
if [ -f ${BASEDIR}/etc/authlib/authdaemonrc ]; then
   echo "found etc/authlib/authdaemonrc"
   CONFCI=0
fi
if [ -f ${BASEDIR}/etc/authlib/authmysqlrc ]; then
   echo "found etc/authlib/authmysqlrc"
   CONFCI=0
fi
if [ $CONFCI -ne 0 ]; then
   echo "CONFIGURE/configure-ci.pl wird ausgefhrt"
   ${BASEDIR}/CONFIGURE/configure-ci.pl
else 
   echo "CONFIGURE/configure-ci.pl wird nicht ausgefhrt."
fi

start_services3

#---------------------------|
# register rssh
#---------------------------|

echo "| "
echo "| rssh wird als shell registriert"
echo "| "
grep -q rssh /etc/shells || echo ${BASEDIR}/bin/rssh >> /etc/shells

#---------------------------|
#  time critical end
#---------------------------|

echo "| "
echo "| Das Update der Serverumgebung (SE) ist abgeschlossen"
echo "| Bitte prfen Sie, ob alle Dienste wieder angelaufen sind,"
echo "| insbesondere solche, die ber /etc/rc.d (z. B. imap) ge-"
echo "| startet werden."
echo "| "

echo "| "
echo "| clamav-Datenbank wird initialisiert/aktualisiert."
echo "| "

chown -hR simscan ${BASEDIR}/share/clamav
rm -fr ${BASEDIR}/share/clamav/daily.inc
${BASEDIR}/CONFIGURE/configure-cl.sh
rm -f /tmp/clamd
rm -f /tmp/clamd.socket
killall -9 clamd

echo "| "
echo "| simscan cdb files werden initialisiert/aktualisiert."
echo "| "

${BASEDIR}/qmail/bin/simscanmk -g

if [ -d /var/qmail/simcontrol-msa ]; then
   CONTROLDIR=/var/qmail/simcontrol-msa \
   ${BASEDIR}/qmail/bin/simscanmk -g
fi

if [ "$SERIES" != "1" ]; then # skip this in SE-1 starting with version 0.340
   if [ -d ${BASEDIR}/htdocs/roundcubemail -a ! -f \
           ${BASEDIR}/htdocs/roundcubemail/config/db.inc.php -a ! -f \
           ${BASEDIR}/htdocs/roundcubemail/config/config.inc.php ]; then
      echo "| "
      echo "| ROUNDCUBEMAIL wird eingerichtet."
      echo "| "
      ${BASEDIR}/CONFIGURE/roundcube.pl
   else
      echo "| "
      echo "| ROUNDCUBEMAIL wird upgegradet."
      echo "| "
      $T1523594 
      $T1524289
   fi
else
   echo "|"
   echo "| Series 1 no longer supports roundcubemail"
   echo "|"
fi

if [ $CVERSION -ge 0325 ]; then
   case $SESN in
      1 | 2 | 3)
         echo "|"
         echo "| phpMyAdmin and phpPgAdmin no longer available, use \$hostname/adminer"
         echo "|"
         ;;
      *)
         echo "|"
         echo "| phpMyAdmin < 4.8.4 and phpPgAdmin no longer available, use \$hostname/adminer"
         echo "|"
         if [ ! -f ${BASEDIR}/htdocs/phpMyAdmin/config.inc.php ]; then
            echo "|"
            echo "| phpMyAdmin 4.8.4: initializing config.ini.php: blowfish_secret"
            echo "|"
            touch ${BASEDIR}/htdocs/phpMyAdmin/config.inc.php
            chown root:phpmyadm ${BASEDIR}/htdocs/phpMyAdmin/config.inc.php
            chmod 0640 ${BASEDIR}/htdocs/phpMyAdmin/config.inc.php
            blowfish_secret=$(dd if=/dev/urandom bs=24 count=1 2>/dev/null | base64 2>/dev/null)
            cat > ${BASEDIR}/htdocs/phpMyAdmin/config.inc.php <<_
<?php
// auto generated by se-update.sh
\$cfg['blowfish_secret'] = '$blowfish_secret';
_
         fi
   esac
fi

echo "| "
echo "| fixing DocumentRoot"
echo "| "
(
mkdir -p ${BASEDIR}/htdocs/forbidden && chmod 700 ${BASEDIR}/htdocs/forbidden
cd /usr/local/pd-admin2/conf
grep 'DocumentRoot /usr/local/pd-admin2/htdocs/forbidden' httpd.conf-template || {

cat <<'__' > /usr/src/docroot.patch
--- httpd.conf-template-1       2013-09-17 15:35:58.231317942 +0200
+++ httpd.conf-template         2013-10-14 17:29:28.363196341 +0200
@@ -8,7 +8,7 @@
  ServerName $$SERVERNAME
  ServerTokens Prod
  ServerRoot /usr/local/pd-admin2
-DocumentRoot /usr/local/pd-admin2/htdocs
+DocumentRoot /usr/local/pd-admin2/htdocs/forbidden
  HostnameLookups off
  LogLevel Error 
__
patch -F3 < /usr/src/docroot.patch
/opt/pdadmin/bin/httpd_vhosts.pl
}
)

if test -f "/var/qmail/plugins/chkauth.sh" && 
   head -1 "/var/qmail/plugins/chkauth.sh" | 
   grep -q '^#!/bin/sh'; then
   F="`mktemp`"
   echo '#!/bin/bash' > "$F"
   tail -n +2 "/var/qmail/plugins/chkauth.sh" >> "$F"
   cat "$F" > "/var/qmail/plugins/chkauth.sh"
   rm "$F"
fi

   if test -f "/var/qmail/bin/dk-filter"; then
      ${BASEDIR}/CONFIGURE/configure-dkim.sh -er
      echo "| "
      echo "| DKIM activated"
      echo "| "
   fi

   echo "| "
   echo "| se-update.sh done"
   echo "| "

   if [ "$DOVECOT_MIGRATION" == "1" ]; then
      if ! test -e /service/dovecot -o -e /service/dovecot22; then
         echo "| "
         echo "| Migration Courier IMAP ----> Dovecot"
         echo "| "
         wget download.pd-admin.de/fix-maildirsize.pl
         perl fix-maildirsize.pl
         wget -N download.pd-admin.de/ci2dc22.sh
         bash ci2dc22.sh
      fi
   else
      echo "skipped migration to dovecot (-c)"
   fi

   if [ -d "/service/dovecot22/" ]; then
      echo "| "
      echo "| regenerating dovecot configuration"
      echo "| "
      ${BASEDIR}/CONFIGURE/dovecot-2.2.sh
      svc -du /service/dovecot22/
   fi

   if [ -e "${BASEDIR}/CONFIGURE/add-roundcubemail-plugin.php" ]
   then
      echo "| "
      echo "| roundcubemail: twofactor_gauthenticator"
      echo "| "
      ${BASEDIR}/CONFIGURE/add-roundcubemail-plugin.php twofactor_gauthenticator
   fi
}

LLOG=log.`date +%Y-%m-%dT%H:%M:%S`
if [ $? -ne 0 ]; then
   echo "date function error"
   exit 2
fi

performe 2>&1 | tee $LLOG
r=${PIPESTATUS[0]}
chmod go= $LLOG
exit $r
